@@ -19,7 +19,7 @@ import os
 import os.path
 
 PACKAGE    = "sabayon"
-VERSION    = "2.25.0"
+VERSION    = "2.27.91"
 LOCALEDIR = "/usr/local/share/locale"
 DATADIR    = "/usr/local/share"
 SYSCONFDIR = "/usr/local/etc"
@@ -36,8 +36,8 @@ APPLY_TOOL_ARGV   = [ os.path.join (SBINDIR,    "sabayon-apply") ]
 # FIXME: this is too big a hammer
 CLOBBER_USER_PROCESSES_ARGV = [ "/usr/bin/pkill", "-u" ]
 
-# No. of seconds to wait for USR1 from Xnest before giving up
-XNEST_USR1_TIMEOUT = 10
+# No. of seconds to wait for USR1 from Xephyr before giving up
+XEPHYR_USR1_TIMEOUT = 10
 
 #
 # This is the user which we run the prototype sessions
@@ -52,7 +52,7 @@ DEFAULT_SHELL = "/bin/bash"
 NOLOGIN_SHELL = "/sbin/nologin"
 SKEL_HOMEDIR  = "/etc/skel"
 
-XNEST_ARGV   = [ "/usr/bin/Xnest", "-terminate", "-audit", "0", "-nolisten", "tcp" ]
+XEPHYR_ARGV  = [ "/usr/bin/Xephyr", "-terminate", "-audit", "0", "-nolisten", "tcp" ]
 SESSION_ARGV = [ "/etc/gdm/Xsession", "gnome" ]
 USERMOD_ARGV = [ "/usr/sbin/usermod" ]
 
@@ -65,6 +65,7 @@ PANEL_KEY_BASE = "/apps/panel"
 #
 PASSTHROUGH_ENVIRONMENT = [
     "SABAYON_DEBUG",
+    "DEBUG_WHILE_IN_SABAYON",
     "LANG",
     "LINGUAS",
     "LC_CTYPE",
@@ -104,6 +105,8 @@ FILES_TO_IGNORE = [
 DIRECTORIES_TO_IGNORE = [
     ".beagle",
     ".cache",
+    ".ccache",
+    ".dbus",
     ".fontconfig",
     ".gconf",
     ".gconf.xml.defaults",
@@ -112,8 +115,31 @@ DIRECTORIES_TO_IGNORE = [
     ".gconfd",
     ".gstreamer-0.8",
     ".gstreamer-0.10",
+    ".gvfs",
+    ".local/share/tracker",
+    ".metacity/sessions",
+    ".profile_cache",
+    ".pulse",
+    ".thumbnails",
+    ".wapi",
+    ".Trash"
+]
+
+#
+# Directories which we don't want to add when saving profile
+# 
+DIRECTORIES_TO_IGNORE_PROFILE = [
+    ".beagle",
+    ".cache",
+    ".ccache",
+    ".dbus",
+    ".fontconfig",
+    ".gconfd",
+    ".gstreamer-0.8",
+    ".gstreamer-0.10",
+    ".gvfs",
+    ".local/share/tracker",
     ".metacity/sessions",
-    ".nautilus/metafiles",
     ".profile_cache",
     ".pulse",
     ".thumbnails",
