@@ -24,24 +24,27 @@ import errno
 import gobject
 import gconf
 import fnmatch
+import subprocess
 
 try:
     import userprofile
     import storage
     import util
     from config import *
+    import debuglog
 except:
     from sabayon import userprofile
     from sabayon import storage
     from sabayon import util
     from sabayon.config import *
+    from sabayon import debuglog
 
 def dprint (fmt, *args):
-    util.debug_print (util.DEBUG_GCONFSOURCE, fmt % args)
+    debuglog.debug_log (False, debuglog.DEBUG_LOG_DOMAIN_GCONF_SOURCE, fmt % args)
 
 # gconf_engine_associate_schema() isn't wrapped
 def associate_schema (config_source, key, schema_key):
-    os.system ("gconftool-2 --config-source='%s' --apply-schema %s %s" % (config_source, schema_key, key))
+    subprocess.call (["gconftool-2", "--config-source=%s" % (config_source), "--apply-schema", "%s" % (schema_key), "%s" % (key)])
 
 def copy_tree (src_client, dst_client, dir):
     for entry in src_client.all_entries (dir):
@@ -122,7 +125,7 @@ class GConfSource (userprofile.ProfileSource):
 
         self.storage              = storage
         self.home_dir             = util.get_home_dir ()
-        self.client               = None
+        self.gconf_client         = None
         self.notify_id            = 0
         self.defaults_client      = None
         self.mandatory_client     = None
@@ -195,7 +198,7 @@ class GConfSource (userprofile.ProfileSource):
             return
         
         def handle_notify (client, cnx_id, entry, self):
-            dprint ("Got GConf notification on '%s'", entry.key)
+            dprint ("Got GConf notification on '%s', value='%s', default=%s", entry.key, entry.value, entry.get_is_default ())
             
             for ignore_pattern in GCONF_KEYS_TO_IGNORE:
                 if fnmatch.fnmatchcase (entry.key, ignore_pattern):
@@ -210,19 +213,19 @@ class GConfSource (userprofile.ProfileSource):
             self.emit_change (GConfChange (self, entry.key, value))
 
         # Only monitor for changes in the user settings database
-        (self.client, address) = get_client_and_address_for_path (os.path.join (self.home_dir, ".gconf"))
-        self.client.add_dir ("/", gconf.CLIENT_PRELOAD_RECURSIVE)
-        self.notify_id = self.client.notify_add ("/", handle_notify, self)
+        (self.gconf_client, address) = get_client_and_address_for_path (os.path.join (self.home_dir, ".gconf"))
+        self.gconf_client.add_dir ("/", gconf.CLIENT_PRELOAD_RECURSIVE)
+        self.notify_id = self.gconf_client.notify_add ("/", handle_notify, self)
 
     def stop_monitoring (self):
         """Stop monitoring for GConf changes."""
         if self.notify_id == 0:
             return
 
-        self.client.notify_remove (self.notify_id)
+        self.gconf_client.notify_remove (self.notify_id)
         self.notify_id = 0
-        self.client.remove_dir ("/")
-        self.client = None
+        self.gconf_client.remove_dir ("/")
+        self.gconf_client = None
 
     def sync_changes (self):
         """Ensure that all committed changes are saved to disk."""
@@ -230,7 +233,7 @@ class GConfSource (userprofile.ProfileSource):
         # FIXME: it would be nicer if we just wrote directly
         #        to the defaults and mandatory sources
         #dprint ("Shutting down gconfd in order to sync changes to disk")
-        #os.system ("gconftool-2 --shutdown")
+        #subprocess.call (["gconftool-2", "--shutdown"])
         if self.defaults_client:
             self.defaults_client.suggest_sync();
         if self.mandatory_client:
@@ -318,13 +321,13 @@ class GConfSource (userprofile.ProfileSource):
         #        if we could guarantee that there wasn't a gconfd already
         #        running.
         dprint ("Shutting down gconfd so it kill pick up new paths")
-        os.system ("gconftool-2 --shutdown")
+        subprocess.call (["gconftool-2", "--shutdown"])
 
     def add_gconf_notify (self, key, handler, data):
-        return self.client.notify_add (key, handler, data)
+        return self.gconf_client.notify_add (key, handler, data)
 
     def remove_gconf_notify (self, id):
-        return self.client.notify_remove (id)
+        return self.gconf_client.notify_remove (id)
 
     def get_gconf_key_is_mandatory (self, key):
         (client, address) = self.get_committing_client_and_address (True)
@@ -333,13 +336,26 @@ class GConfSource (userprofile.ProfileSource):
             return True
         return False
 
+    def set_value (self, key, gconf_value, mandatory):
+        change = GConfChange (self, key, gconf_value)
+        change.set_mandatory (mandatory)
+        self.gconf_client.set (key, gconf_value)
+        self.emit_change (change)
+
     def set_gconf_boolean (self, key, value, mandatory):
         gconf_value = gconf.Value (gconf.VALUE_BOOL)
         gconf_value.set_bool (value)
-        change = GConfChange (self, key, gconf_value)
-        change.set_mandatory (mandatory)
-        self.client.set_bool (key, value)
-        self.emit_change (change)
+        self.set_value (key, gconf_value, mandatory);
+
+    def set_gconf_int (self, key, value, mandatory):
+        gconf_value = gconf.Value (gconf.VALUE_INT)
+        gconf_value.set_int (value)
+        self.set_value (key, gconf_value, mandatory);
+
+    def set_gconf_string (self, key, value, mandatory):
+        gconf_value = gconf.Value (gconf.VALUE_STRING)
+        gconf_value.set_string (value)
+        self.set_value (key, gconf_value, mandatory);
 
     def set_gconf_list (self, key, list_type, value, mandatory):
         gconf_value = gconf.Value (gconf.VALUE_LIST)
@@ -353,10 +369,7 @@ class GConfSource (userprofile.ProfileSource):
             list.append (item_value)
         gconf_value.set_list_type (list_type)
         gconf_value.set_list (list)
-        change = GConfChange (self, key, gconf_value)
-        change.set_mandatory (mandatory)
-        self.client.set_list (key, list_type, value)
-        self.emit_change (change)
+        self.set_value (key, gconf_value, mandatory);
 
 gobject.type_register (GConfSource)
 
@@ -383,13 +396,13 @@ def run_unit_tests ():
         pass
 
     # Need to shutdown the daemon to ensure its not using stale paths
-    os.system ("gconftool-2 --shutdown")
+    subprocess.call (["gconftool-2", "--shutdown"])
     time.sleep (1)
 
     # Make sure there's no stale keys from a previous run
     # FIXME: gconf_client_recursive_unset() has no wrapping
-    # source.client.recursive_unset ("/tmp/test-gconfprofile")
-    os.system ("gconftool-2 --recursive-unset /tmp/test-gconfprofile")
+    # source.gconf_client.recursive_unset ("/tmp/test-gconfprofile")
+    subprocess.call (["gconftool-2", "--recursive-unset", "/tmp/test-gconfprofile"])
     time.sleep (1)
 
     global changes
@@ -410,17 +423,17 @@ def run_unit_tests ():
         while main_loop.get_context ().pending ():
             main_loop.get_context ().iteration (False)
         
-    source.client.set_bool ("/tmp/test-gconfprofile/t1", True)
+    source.gconf_client.set_bool ("/tmp/test-gconfprofile/t1", True)
     poll (main_loop)
-    source.client.set_bool ("/tmp/test-gconfprofile/t1", False)
+    source.gconf_client.set_bool ("/tmp/test-gconfprofile/t1", False)
     poll (main_loop)
-    source.client.set_bool ("/tmp/test-gconfprofile/t2", True)
+    source.gconf_client.set_bool ("/tmp/test-gconfprofile/t2", True)
     poll (main_loop)
-    source.client.set_int ("/tmp/test-gconfprofile/t3", 3)
+    source.gconf_client.set_int ("/tmp/test-gconfprofile/t3", 3)
     poll (main_loop)
     
     source.stop_monitoring ()
-    source.client = gconf.client_get_default ()
+    source.gconf_client = gconf.client_get_default ()
     
     assert len (changes) == 4
     assert changes[3].key == "/tmp/test-gconfprofile/t3"
@@ -432,8 +445,8 @@ def run_unit_tests ():
     assert changes[1].key == "/tmp/test-gconfprofile/t1"
     assert changes[0].key == "/tmp/test-gconfprofile/t1"
 
-    # source.client.recursive_unset ("/tmp/test-gconfprofile")
-    os.system ("gconftool-2 --recursive-unset /tmp/test-gconfprofile")
+    # source.gconf_client.recursive_unset ("/tmp/test-gconfprofile")
+    subprocess.call (["gconftool-2", "--recursive-unset", "/tmp/test-gconfprofile"])
     
     source.sync_changes ()
     source.apply (False)
@@ -444,16 +457,16 @@ def run_unit_tests ():
     # We need to clear the cache because GConfClient doesn't know
     # some new sources have been added to the sources stack so it
     # won't see the value we put in the mandatory source
-    source.client.clear_cache ()
+    source.gconf_client.clear_cache ()
     
-    entry = source.client.get_entry ("/tmp/test-gconfprofile/t3", "", False)
+    entry = source.gconf_client.get_entry ("/tmp/test-gconfprofile/t3", "", False)
     assert entry.value
     assert entry.value.type == gconf.VALUE_INT
     assert entry.value.get_int () == 3
     assert not entry.get_is_default ()
     assert entry.get_is_writable ()
     
-    entry = source.client.get_entry ("/tmp/test-gconfprofile/t2", "", False)
+    entry = source.gconf_client.get_entry ("/tmp/test-gconfprofile/t2", "", False)
     assert entry.value
     assert entry.value.type == gconf.VALUE_BOOL
     assert entry.value.get_bool () == True
@@ -462,7 +475,7 @@ def run_unit_tests ():
 
     # Shutdown the daemon and remove the path files so we don't screw
     # too much with the running session
-    os.system ("gconftool-2 --shutdown")
+    subprocess.call (["gconftool-2", "--shutdown"])
     time.sleep (1)
 
     os.remove (os.path.join (util.get_home_dir (), ".gconf.path.defaults"))
